package info.textgrid.middleware;



import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.ListIdentifiersType;
import info.textgrid.middleware.oaipmh.RequestType;

import java.text.ParseException;

import javax.xml.datatype.DatatypeConfigurationException;

import org.apache.commons.logging.LogFactory;
import org.elasticsearch.action.search.SearchRequestBuilder;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;



/**
 * Class to build the Element for a ListIdentifiers request
 * 
 * @author Maximilian Brodhun, SUB
 * @version 1.0
 * @since 20.02.2014
 */

public class IdentifierListDeliverer {

	private static org.apache.commons.logging.Log	log		= LogFactory
																	.getLog(IdentifierListDeliverer.class);
	private String									datestamp;
	private boolean									idExist	= true;
	private OAI_ESClient							oaiEsClient;

	private boolean									textgrid;
	private boolean									dariah;

	private String[]								identifierListFields;  //Fields for the elasticsearch request
	private String									rangeField; 		// Field for the optional range query 
	private String									formatToFilter;		// just necessary in textgrid case to just get the editions
	private String									formatField;		// just necessary in textgrid to just get the editions
	private String									dateOfObjectCreation; // Field for the object creation in the repository 
	private String									repositoryObjectURIPrefix; //
	private String									identifierField;
	
	//private String 									set;

	/**
	 * In OAIPMH a ListIdentifiers request is answered by responding the
	 * datestamp and the identifier of an object Both will be responded in a
	 * header element. This function build the header element
	 * 
	 * @param datestamp
	 *            taken from the elasticSearch index (created element)
	 * @param identifier
	 *            taken from the elasticSearch index (textgridUri)
	 * @return the ListIdentifierElement with the responded header element
	 */

	public IdentifierListDeliverer(OAI_ESClient oaiEsClient, boolean textgrid,
			boolean dariah) {
		this.textgrid = textgrid;
		this.dariah = dariah;
		this.oaiEsClient = oaiEsClient;
		
	}
	
	
	// FIXME Delete this function and check for empty set string in setListIdentifierHeaderWithSet
	
	private ListIdentifiersType setListIdentifierHeader(String datestamp,
			String identifier, ListIdentifiersType lit, String set) {
		
		HeaderType header = new HeaderType();
		
		header.setDatestamp(datestamp);
		header.setIdentifier(identifier);
		
		if(set!=null){
			header.getSetSpec().add(set);
		}
		
		lit.getHeader().add(header);

		return lit;
	}


	/**
	 * To get the required values for the ListIdentifiers request this function
	 * will ask ElasticSearch for a specific textgridUri the values "created"
	 * and "textgridUri".
	 * 
	 * Since the ListIdentifiers request enables the possibility the limit the
	 * request in a specific time interval, it is necessary to perform a range
	 * query on the ElasticSearch index
	 * 
	 * @param from
	 *            : start value for the range query
	 * @param to
	 *            : end value to the range query
	 * @return after calling the function "setListIdentifierHeader" the return
	 *         value is the whole ListIdentifiers element
	 * @throws ParseException
	 */
	
	// FIXME combine it with with getIdentifierListWithSet and check for empty set
	
	
	public ListIdentifiersType processIdentifierList(String from, String to, String set)
			throws ParseException {
		ListIdentifiersType lit = new ListIdentifiersType();
		
		String identifier;
		QueryBuilder query;
		QueryBuilder rangeQuery = QueryBuilders.rangeQuery(rangeField)
				.from(from).to(to);
		
		
		System.out.println("dariah has value: " + dariah);
		System.out.println("set has value: " + set);
		
		if(set!=null){
			//ithis.set=set;
			System.out.println("inside set with value:  " + set);
			
			String queryField = "";
			String valueField = "";
			
			if (this.dariah == true) {
				/*
				 * COLLECTIONREGISTRY_NAMESPACE is the name of the set Category
				 * but no field in ElasticSearch. It has to be queried in field
				 * "metadata.dc:relation".
				 */
				
				queryField = "metadata.dc:relation";
				valueField = set;
				System.out.println("check");
				System.out.println("valueField: " + valueField);
				
			}

			if (this.textgrid == true) {
				String[] setParts = set.split(":");

				if (setParts[0].equals("project")) {
					queryField = "project.value";
					valueField = setParts[1];
				}
			}
				
			QueryBuilder matchQuery = QueryBuilders.matchPhraseQuery(queryField, valueField);
			QueryBuilder boolQuery = QueryBuilders.boolQuery().must(rangeQuery).must(matchQuery);
			
			query = boolQuery;			
		}else {
			query = rangeQuery;
		}
		
		SearchRequestBuilder getListIdentifiersValuesBuilder = oaiEsClient
				.getOaiESClient().prepareSearch(oaiEsClient.getEsIndex())
				.setTypes(oaiEsClient.getEsType())
				.addFields(identifierListFields).setQuery(query)
				.setSize(10000);
		
		SearchResponse listListIdentiferValues = getListIdentifiersValuesBuilder
				.execute().actionGet();
		
		
		if (listListIdentiferValues.getHits().getTotalHits() > 0) {
			System.out.println(listListIdentiferValues.getHits().getTotalHits() + ">" + 0);
			for (SearchHit hit : listListIdentiferValues.getHits().getHits()) {

				if (this.textgrid && hit.getFields().get(formatField).getValue()
						.toString().equals(formatToFilter)) {
					
					System.out.println(hit.getId());
					
					System.out.println("textgrid has value: " + textgrid);
					System.out.println("formatField has value: " + formatField);
					System.out.println("formatToFilter has value: " + formatToFilter);
					
			
					try {

						String datestamp = hit.getFields().get(this.dateOfObjectCreation).getValue().toString();

						System.out.println("datestamp: " + datestamp);
						
						datestamp = OAIPMHUtillities.convertDateFormat(datestamp).toXMLFormat();
						
						System.out.println("datestamp: " + datestamp);
						System.out.println("got the date in xml format");
						
					} catch (ParseException e1) {
						log.debug(e1);
					} catch (DatatypeConfigurationException e1) {
						log.debug(e1);
					}
					
					System.out.println("identifierFIeld has value: " + identifierField);
					
					identifier = hit.getFields().get(identifierField).getValue().toString();

					lit = setListIdentifierHeader(datestamp, identifier, lit, set);
				}

				// FIXME @MAX: REMOVE CODE DUPLICATION!!!!!!!

				if (this.dariah) {
					System.out.println("start with dariah process");
					System.out.println("dariah ist set to: " + dariah);
					datestamp = hit.getFields().get(dateOfObjectCreation)
							.getValue().toString();
					
					/*try {
						// XMLGregorianCalendar convertedDateFormat=
						// OAIPMHUtillities.convertDateFormat(datestamp);
						System.out.println("check datestamp");
						datestamp = OAIPMHUtillities.convertDateFormat(
								datestamp).toXMLFormat();
						System.out.println("check datestamp");
					} catch (DatatypeConfigurationException e) {
						log.debug(e);
					}*/
					try {

						String datestamp = hit.getFields()
								.get(this.dateOfObjectCreation).getValue()
								.toString();

						datestamp = OAIPMHUtillities.convertDateFormat(
								datestamp).toXMLFormat();
					} catch (ParseException e1) {
						log.debug(e1);
					} catch (DatatypeConfigurationException e1) {
						log.debug(e1);
					}
					
					// FIXME identifierPrefix to identifierField
					
					identifier = hit.getFields().get(identifierField)
							.getValue().toString();
					System.out.println(identifier);
					lit = setListIdentifierHeader(datestamp, identifier, lit, set);
				}
			}

		} else {

			lit = null;
		}
		System.out.println("here we are");
		return lit;
	}

	public ListIdentifiersType getLit(ListIdentifiersType lit) {
		return lit;
	}

	/**
	 * checking the request if all necessary values are set or not allowed
	 * values are not set
	 * 
	 * @param request
	 *            the initially request from the OAIPMG client
	 * @return a boolean indicating the correctness of the request
	 */

	public boolean requestChecker(RequestType request) {
		boolean requestCheck;

		if (request.getMetadataPrefix() == null
				|| request.getIdentifier() != null) {

			requestCheck = false;
		} else {
			requestCheck = true;
		}

		return requestCheck;

	}

	public void setFormatToFilter(String formatToFilter) {
		this.formatToFilter = formatToFilter;
	}

	public void setFormatField(String formatField) {
		this.formatField = formatField;
	}

	public String getDateOfObjectCreation() {
		return dateOfObjectCreation;
	}

	public void setDateOfObjectCreation(String dateOfObjectCreation) {
		this.dateOfObjectCreation = dateOfObjectCreation;
	}

	public void setRepositoryObjectURIPrefix(String repositoryObjectURIPrefix) {
		this.repositoryObjectURIPrefix = repositoryObjectURIPrefix;
	}

	public String getRepositoryObjectURIPrefix() {
		return repositoryObjectURIPrefix;
	}

	public void setIdExist(boolean validItem) {
		idExist = validItem;
	}

	public boolean getIdExist() {
		return idExist;
	}

	public String[] getIdentifierListFields() {
		return identifierListFields;
	}

	public void setIdentifierListFields(String[] identifierListFields) {
		this.identifierListFields = identifierListFields;
	}

	public String getFieldForRange() {
		return rangeField;
	}

	public void setFieldForRange(String fieldForRange) {
		this.rangeField = fieldForRange;
	}
	
	public String getIdentifierField() {
		return identifierField;
	}

	public void setIdentifierField(String identifierPrefix) {
		this.identifierField = identifierPrefix;
	}
}
