package info.textgrid.middleware;


import info.textgrid.middleware.oaipmh.ListMetadataFormatsType;
import info.textgrid.middleware.oaipmh.MetadataFormatType;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

/**
 * OAIPMHUtillities is responsible to implement all necessary functions to implement the arguments of a request
 * 
 * @author Maximilian Brodhun: SUB
 * @version 1.0
 * @since 29.01.2014
 *
 */

public class OAIPMHUtillities {
	
	/**
	 * Since textGrid just supports dc the only returned MetadataFormat is dublinCore 
	 * @return
	 */
	
	public MetadataFormatType setMetadataFormats(){
		MetadataFormatType tgDublinCore = new MetadataFormatType();
		
		tgDublinCore.setMetadataNamespace("http://www.openarchives.org/OAI/2.0/oai_dc/");
		tgDublinCore.setMetadataPrefix("oai:dc");
		tgDublinCore.setSchema("http://www.openarchives.org/OAI/2.0/oai_dc.xsd");
		
		return tgDublinCore;
	}
	
	/**
	 * producing the list of all metadata formats
	 * @return tgRepMetadaFormats
	 */
	
	public ListMetadataFormatsType setMetadataFormatList(){
		
		ListMetadataFormatsType tgRepMetadataFormats = new ListMetadataFormatsType();

		MetadataFormatType tgDublinCore = setMetadataFormats();
		
		tgRepMetadataFormats.getMetadataFormat().add(tgDublinCore );
	
		return tgRepMetadataFormats;
	}
	
	/**
	 * calculating the acutuall date and give it back as XMLGregorianCalendar
	 * @return now
	 * @throws DatatypeConfigurationException
	 */
	
	public static XMLGregorianCalendar getXMLGregorianCalendarNow() throws DatatypeConfigurationException{
		GregorianCalendar gregorianCalendar = new GregorianCalendar();
	        DatatypeFactory datatypeFactory = DatatypeFactory.newInstance();
	        XMLGregorianCalendar now = 
	            datatypeFactory.newXMLGregorianCalendar(gregorianCalendar);
	        return now;
	}
	  
	/**
	 * Converting a given string representing a date value into the date format required for OAIPMH 
	 * and give it back as XMLGregorianCalendar
	 *  
	 * @param originalDateTimeString
	 * @return xmlCal
	 * @throws ParseException
	 * @throws DatatypeConfigurationException
	 */
	public static XMLGregorianCalendar convertDateFormat(String originalDateTimeString) throws ParseException, DatatypeConfigurationException{
		   
		SimpleDateFormat tgItemTime = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.S");
		Date date = tgItemTime.parse(originalDateTimeString);
		SimpleDateFormat outFormatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
		outFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
		String output2 = outFormatter.format(date);
		XMLGregorianCalendar xmlCal = DatatypeFactory.newInstance().newXMLGregorianCalendar(output2);
		    
		return xmlCal;
	}
	
	/**
	 * Converting a given string representing a date value into the date format required for OAIPMH
	 * and give it back as string
	 * 
	 * @param originalDateTimeString
	 * @return dateOutputAsString 
	 * @throws ParseException
	 */
	
	public static String datestampAsString(String originalDateTimeString) throws ParseException{
		   
		SimpleDateFormat tgItemTime = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.S");
		Date date = tgItemTime.parse(originalDateTimeString);
		SimpleDateFormat outFormatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
		outFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
		String dateOutputAsString = outFormatter.format(date);
				    
		return dateOutputAsString;
	}
	
	
}
