package info.textgrid.middleware;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.datatype.DatatypeConfigurationException;

import org.apache.commons.logging.LogFactory;
import org.elasticsearch.action.get.GetRequestBuilder;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.action.search.SearchRequestBuilder;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;

import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.ListRecordsType;
import info.textgrid.middleware.oaipmh.MetadataType;
import info.textgrid.middleware.oaipmh.OaiDcType;
import info.textgrid.middleware.oaipmh.ObjectFactory;
import info.textgrid.middleware.oaipmh.RecordType;
import info.textgrid.middleware.oaipmh.RequestType;

/**
 ** 
 * TODO Care about repeatable fields as done in TITLE!
 */

public class RecordListDeliverer {

	// private static final String BIBCIT_BIBID =
	// "edition.source.bibliographicCitation.bibidentifier";

	private String									set;

	// **
	// STATICS
	// **

	private static org.apache.commons.logging.Log	log				= LogFactory
																			.getLog(RecordDeliverer.class);

	// **
	// CLASS VARIABLES
	// **

	private OAI_ESClient							oaiEsClient;
	private String[]								fields;
	private String[]								workFields;
	private boolean									textgrid;
	private boolean									dariah;
	private String									formatField;
	private String									formatToFilter;
	private String									dateOfObjectCreation;
	private String									relationToFurtherMetadataObject;
	private String									repositoryObjectURIPrefix;
	private String									rangeField;
	private boolean									foundItems;


	// **
	// DC-Field Lists
	// **

	private List<String>							contributors	= new ArrayList<String>();
	private List<String>							coverages		= new ArrayList<String>();
	private List<String>							creators		= new ArrayList<String>();
	private List<String>							dates			= new ArrayList<String>();
	private List<String>							descriptions	= new ArrayList<String>();
	private List<String>							formats			= new ArrayList<String>();
	private List<String>							identifiers		= new ArrayList<String>();
	private List<String>							languages		= new ArrayList<String>();
	private List<String>							publishers		= new ArrayList<String>();
	private List<String>							relations		= new ArrayList<String>();
	private List<String>							rights			= new ArrayList<String>();
	private List<String>							sources			= new ArrayList<String>();
	private List<String>							subjects		= new ArrayList<String>();
	private List<String>							titles			= new ArrayList<String>();
	private List<String>							types			= new ArrayList<String>();

	private String[]								contributorList;
	private String[]								coverageList;
	private String[]								creatorList;
	private String[]								dateList;
	private String[]								descriptionList;
	private String[]								formatList;
	private String[]								identifierList;
	private String[]								languageList;
	private String[]								publisherList;
	private String[]								relationList;
	private String[]								relationForWorkList;
	private String[]								rightList;
	private String[]								sourceList;
	private String[]								subjectList;
	private String[]								titleList;
	private String[]								typeList;

	/**
	 * @param oaiEsClient
	 */
	public RecordListDeliverer(OAI_ESClient oaiEsClient, String[] fields,
			String[] workFields, boolean textgrid, boolean dariah) {
		this.oaiEsClient = oaiEsClient;
		this.fields = fields;
		this.workFields = workFields;
		this.textgrid = textgrid;
		this.dariah = dariah;
	}

	/**
	 * the furtherDCElementes fetches the specified fields from the work object.
	 * This function is called from the function fetchFields
	 * 
	 * @param id
	 * @param esClient
	 * @throws UnsupportedEncodingException
	 */
	public GetResponse furtherDCElements(String id, OAI_ESClient esClient) {

		try {
			id = URLDecoder.decode(id, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		
		GetRequestBuilder getWorkValues = esClient.getOaiESClient()
				.prepareGet().setIndex(esClient.getEsIndex())
				.setType(esClient.getEsType()).setId(id)
				.setFields(this.workFields);

			
		GetResponse responseWorkValues = getWorkValues.execute().actionGet();
		

		/*if (this.dariah == true
				&& responseWorkValues.getField("metadata.dc:title") != null) {
			this.collectionTitle = responseWorkValues
					.getField("metadata.dc:title").getValue().toString();
		}*/

		return responseWorkValues;
	}

	/**
	 * <p>
	 * The fetchFields function filters all edition and fetch the specified
	 * fields
	 * </p>
	 * 
	 * @param query
	 * @param recordList
	 */
	public void fetchFields(QueryBuilder query, ListRecordsType recordList) {

		SearchRequestBuilder getRecordList = this.oaiEsClient.getOaiESClient()
				.prepareSearch(this.oaiEsClient.getEsIndex())
				.setTypes(this.oaiEsClient.getEsType()).addFields(this.fields)
				.setQuery(query).setSize(1000);

		SearchResponse getRecordListItems = getRecordList.execute().actionGet();

		if (getRecordListItems != null && getRecordListItems.getHits().totalHits() > 0) {
			setFoundItems(true);

			for (SearchHit hit : getRecordListItems.getHits().getHits()) {
				
				// Only if we have a search response AND a hit field map...
				if (hit != null && hit.getFields() != null) {
					
					// The response should also contains the editions

					// if (hit.getFields().get(formatField) == null) {
					// System.out.println("NULL: "
					// + hit.getFields().get("metadata.dc:identifier")
					// .values().get(0));
					// }
					//
					// if (hit.getFields().get(formatField).values().get(0)
					// .toString().equals(formatToFilter)) {

					// The date stamp from textgrid is in not compatible format
					// so it has to be transformed
					
					try {
						String datestamp = hit.getFields().get(this.dateOfObjectCreation).getValue().toString();
						datestamp = OAIPMHUtillities.convertDateFormat(datestamp).toXMLFormat();
					} catch (ParseException e1) {
						log.debug(e1);
					} catch (DatatypeConfigurationException e1) {
						log.debug(e1);
					}
					// URI of the work Object to get further DC-Fields (only
					// necessary if metadata for the object
					// is in an other object than just the edition

					// TextGrid search!
					
					if (this.textgrid == true && hit.getFields().get(formatField).values()
							.get(0).toString().equals(formatToFilter)) {
						System.out.println("textgriduri is: " + hit.getId());
						String workUri = hit.getFields().get(this.relationToFurtherMetadataObject).values().get(0).toString()
								.substring(this.repositoryObjectURIPrefix.length());
					
						if (!workUri.endsWith(".0")) {
							workUri = workUri.concat(".0");
							System.out.println("workuri is: " + workUri);	
						}
						
						putContentIntoDCFieldLists(hit,	furtherDCElements(workUri, this.oaiEsClient));
						buildRecord(recordList);
						listClearer();
					}

					if (this.dariah == true) {

						putContentIntoDCFieldLists2(hit, furtherDCElements(hit.getId(), this.oaiEsClient));
						buildRecord(recordList);
						listClearer();
					}

				}
			}
		} else {
			setFoundItems(false);
		}
	}

	/**
	 * @param hit
	 * @param responseWorkValues
	 */
	public void putContentIntoDCFieldLists(SearchHit hit,
			GetResponse responseWorkValues) {
		System.out.println("Check");
		this.contributors = DublinCoreFieldLoader.setContributor(hit,this.contributorList);
		this.coverages = DublinCoreFieldLoader.setCoverage(responseWorkValues,this.coverageList);
		this.creators = DublinCoreFieldLoader.setCreator(responseWorkValues,this.creatorList);
		this.dates = DublinCoreFieldLoader.setDate(responseWorkValues,this.dateList);
		this.descriptions = DublinCoreFieldLoader.setDescription(responseWorkValues, this.descriptionList);
		this.formats = DublinCoreFieldLoader.setFormat(hit, this.formatList);
		this.identifiers = DublinCoreFieldLoader.setIdentifier(hit,this.identifierList);
		this.languages = DublinCoreFieldLoader.setLanguage(hit,this.languageList);
		this.publishers = DublinCoreFieldLoader.setPublisher(hit,this.publisherList);
		this.relations = DublinCoreFieldLoader.setRelation(hit,this.relationList);
		this.relations = DublinCoreFieldLoader.setRelationForWork(responseWorkValues, this.relationForWorkList);
		this.rights = DublinCoreFieldLoader.setRights(hit, this.rightList);
		this.sources = DublinCoreFieldLoader.setSources(hit, this.sourceList);
		this.subjects = DublinCoreFieldLoader.setSubject(hit, this.subjectList);
		this.titles = DublinCoreFieldLoader.setTitle(hit, this.titleList);
		this.types = DublinCoreFieldLoader.setType(responseWorkValues,this.typeList);

	}

	/**
	 * @param hit
	 * @param responseWorkValues
	 */
	public void putContentIntoDCFieldLists2(SearchHit hit,
			GetResponse responseWorkValues) {

		this.contributors = DublinCoreFieldLoader.setContributor(hit,this.contributorList);
		this.coverages = DublinCoreFieldLoader.setCoverage(hit,this.coverageList);
		this.creators = DublinCoreFieldLoader.setCreator(hit, this.creatorList);
		this.dates = DublinCoreFieldLoader.setDate(hit, this.dateList);
		this.descriptions = DublinCoreFieldLoader.setDescription(hit,this.descriptionList);
		this.formats = DublinCoreFieldLoader.setFormat(hit, this.formatList);
		this.identifiers = DublinCoreFieldLoader.setIdentifier(hit,this.identifierList);
		this.languages = DublinCoreFieldLoader.setLanguage(hit,	this.languageList);
		this.publishers = DublinCoreFieldLoader.setPublisher(hit, this.publisherList);
		this.relations = DublinCoreFieldLoader.setRelation(hit,	this.relationList);
		this.rights = DublinCoreFieldLoader.setRights(hit, this.rightList);
		this.sources = DublinCoreFieldLoader.setSources(hit, this.sourceList);
		this.subjects = DublinCoreFieldLoader.setSources(hit, this.subjectList);
		this.titles = DublinCoreFieldLoader.setTitle(hit, this.titleList);
		this.types = DublinCoreFieldLoader.setType(hit, this.typeList);

	}

	/**
	 * @param recordList
	 * @return
	 */
	public RecordType buildRecord(ListRecordsType recordList) {

		ObjectFactory of = new ObjectFactory();
		OaiDcType odt = new OaiDcType();
		DublinCoreBuilder newDbc = new DublinCoreBuilder(odt, of);
		MetadataType metadata = new MetadataType();
		RecordType record = new RecordType();
		// TODO Test if this is unnecessary
		HeaderType recordHeader = new HeaderType();
		
		setDCObject(odt, of);
		
		System.out.println("build Records");
		
		if (!this.identifiers.isEmpty()) {
			recordHeader.setIdentifier(this.identifiers.get(0));
		}
		if (!this.dates.isEmpty()) {
			recordHeader.setDatestamp(this.dates.get(0));
		}

		metadata = newDbc.getDC();
		record.setMetadata(metadata);
		record.setHeader(setHeader(this.set));
		recordList.getRecord().add(record);

		return record;
	}

	/**
	 * @param from
	 * @param to
	 * @param set
	 * @return
	 */
	public ListRecordsType getRecordsWithSet(String from, String to, String set) {

		ListRecordsType recordList = new ListRecordsType();
		QueryBuilder query;
		QueryBuilder rangeQuery = QueryBuilders.rangeQuery(this.rangeField)
				.from(from).to(to);

		if (set != null) {
			this.set = set;

			// NOTE For DARIAH, set contains the complete value of the
			// dc:relation field that points to the DARIAH collection. We do
			// need only the identifier of the set, so that we can query the
			// ESDB so we take the complete content after THE LAST ":" setSpec
			// delimeter.
			String queryField = "";
			String valueField = "";

			System.out.println("set: " + set);

			if (this.dariah == true) {
				/*
				 * COLLECTIONREGISTRY_NAMESPACE is the name of the set Category
				 * but no field in ElasticSearch. It has to be queried in field
				 * "metadata.dc:relation".
				 */
				queryField = "metadata.dc:relation";
				valueField = set;
				// setParts[1] = setParts[1].concat(":" + setParts[2]);

				System.out.println("queryField: " + queryField);
				System.out.println("valueField: " + valueField);
				// System.out.println(setParts[0] + " : " + setParts[1]);
			}

			if (this.textgrid == true) {
				String[] setParts = set.split(":");

				if (setParts[0].equals("project")) {
					queryField = "project.value";
					valueField = setParts[1];
					// setParts[0] = setParts[0].concat(".value");

					System.out.println("queryField: " + queryField);
					System.out.println("valueField: " + valueField);
				}
			}

			QueryBuilder matchQuery = QueryBuilders.matchPhraseQuery(
					queryField, valueField);
			QueryBuilder boolQuery = QueryBuilders.boolQuery().must(rangeQuery)
					.must(matchQuery);

			query = boolQuery;
		} else {
			query = rangeQuery;
		}

		fetchFields(query, recordList);

		return recordList;
	}

	/**
	 * @param set
	 * @return
	 */
	public HeaderType setHeader(String set) {

		HeaderType header = new HeaderType();
		System.out.println("Date content is: " + dates);
		header.setDatestamp(this.dates.get(0));
		header.setIdentifier(this.identifiers.get(0));
		if (set != null) {
			header.getSetSpec().add(DARIAHConstants.COLLECTIONREGISTRY_PREFIX + ":" + set);
			// header.getSetSpec().add(this.collectionTitle);
		}
		return header;
	}

	/**
	 * @param odt
	 * @param of
	 * @return
	 */
	public DublinCoreBuilder setDCObject(OaiDcType odt, ObjectFactory of) {

		DublinCoreBuilder dbc = new DublinCoreBuilder(odt, of);

		dbc.setContributor(this.contributors);
		dbc.setCoverage(this.coverages);
		dbc.setCreator(this.creators);
		dbc.setDate(this.dates);
		dbc.setDescription(this.descriptions);
		dbc.setFormat(this.formats);
		dbc.setIdentifier(this.identifiers);
		dbc.setLanguage(this.languages);
		dbc.setPublisher(this.publishers);
		dbc.setRelation(this.relations);
		dbc.setRights(this.rights);
		dbc.setSource(this.sources);
		dbc.setSubject(this.subjects);
		dbc.setTitle(this.titles);
		dbc.setType(this.types);

		return dbc;
	}

	/**
	 * 
	 */
	public void listClearer() {

		this.contributors.clear();
		this.coverages.clear();
		this.creators.clear();
		this.dates.clear();
		this.descriptions.clear();
		this.formats.clear();
		this.identifiers.clear();
		this.languages.clear();
		this.publishers.clear();
		this.relations.clear();
		this.rights.clear();
		this.sources.clear();
		this.subjects.clear();
		this.titles.clear();
		this.types.clear();
	}

	/**
	 * @param formatToFilter
	 */
	public void setFormatToFilter(String formatToFilter) {
		this.formatToFilter = formatToFilter;
	}

	/**
	 * @param formatField
	 */
	public void setFormatField(String formatField) {
		this.formatField = formatField;
	}

	/**
	 * @return
	 */
	public String getDateOfObjectCreation() {
		return this.dateOfObjectCreation;
	}

	/**
	 * @param dateOfObjectCreation
	 */
	public void setDateOfObjectCreation(String dateOfObjectCreation) {
		this.dateOfObjectCreation = dateOfObjectCreation;
	}

	/**
	 * @return
	 */
	public String getRelationToFurtherMetadataObject() {
		return this.relationToFurtherMetadataObject;
	}

	/**
	 * @param relationToFurtherMetadataObject
	 */
	public void setRelationToFurtherMetadataObject(
			String relationToFurtherMetadataObject) {
		this.relationToFurtherMetadataObject = relationToFurtherMetadataObject;
	}

	/**
	 * @return
	 */
	public String getRepositoryObjectURIPrefix() {
		return this.repositoryObjectURIPrefix;
	}

	/**
	 * @param repositoryObjectURIPrefix
	 */
	public void setRepositoryObjectURIPrefix(String repositoryObjectURIPrefix) {
		this.repositoryObjectURIPrefix = repositoryObjectURIPrefix;
	}

	/**
	 * @return
	 */
	public String getFieldForRange() {
		return this.rangeField;
	}

	/**
	 * @param fieldForRange
	 */
	public void setFieldForRange(String fieldForRange) {
		this.rangeField = fieldForRange;
	}

	/*
	 * Setter for DublinCore Lists
	 */

	/**
	 * @param contributorList
	 */
	public void setContributor(String[] contributorList) {
		this.contributorList = contributorList;
	}

	/**
	 * @param coveragesList
	 */
	public void setCoverage(String[] coveragesList) {
		this.coverageList = coveragesList;
	}

	/**
	 * @param creatorsList
	 */
	public void setCreators(String[] creatorsList) {
		this.creatorList = creatorsList;
	}

	/**
	 * @param datesList
	 */
	public void setDates(String[] datesList) {
		this.dateList = datesList;
	}

	/**
	 * @param descriptionsList
	 */
	public void setDescriptions(String[] descriptionsList) {
		this.descriptionList = descriptionsList;
	}

	/**
	 * @param formatsList
	 */
	public void setFormats(String[] formatsList) {
		this.formatList = formatsList;
	}

	/**
	 * @param identifiersList
	 */
	public void setIdentifiers(String[] identifiersList) {
		this.identifierList = identifiersList;
	}

	/**
	 * @param languagesList
	 */
	public void setLanguages(String[] languagesList) {
		this.languageList = languagesList;
	}

	/**
	 * @param publishersList
	 */
	public void setPublishers(String[] publishersList) {
		this.publisherList = publishersList;
	}

	/**
	 * @param relationsList
	 */
	public void setRelations(String[] relationsList) {
		this.relationList = relationsList;
	}

	/**
	 * @param relationsList
	 */
	public void setRelationsForWork(String[] relationsList) {
		this.relationForWorkList = relationsList;
	}

	/**
	 * @param rightsList
	 */
	public void setRights(String[] rightsList) {
		this.rightList = rightsList;
	}

	/**
	 * @param sourcesList
	 */
	public void setSources(String[] sourcesList) {
		this.sourceList = sourcesList;
	}

	/**
	 * @param subjectsList
	 */
	public void setSubjects(String[] subjectsList) {
		this.subjectList = subjectsList;
	}

	/**
	 * @param titlesList
	 */
	public void setTitles(String[] titlesList) {
		this.titleList = titlesList;
	}

	/**
	 * @param typesList
	 */
	public void setTypes(String[] typesList) {
		this.typeList = typesList;
	}

	/**
	 * @param request
	 * @return
	 */
	public boolean requestChecker(RequestType request) {

		boolean requestCheck;

		if (request.getIdentifier() != null
				|| request.getMetadataPrefix() == null) {

			requestCheck = false;
		} else {
			requestCheck = true;
		}

		return requestCheck;
	}

	/**
	 * @return
	 */
	public boolean isFoundItems() {
		return this.foundItems;
	}

	/**
	 * @param foundItems
	 */
	public void setFoundItems(boolean foundItems) {
		this.foundItems = foundItems;
	}

	/*
	 * public String getCollectionTitle() { return collectionTitle; }
	 * 
	 * public void setCollectionTitle(GetResponse responseWorkValues) {
	 * 
	 * this.collectionTitle =
	 * responseWorkValues.getField("metadata.dc:title").getValue().toString();
	 * 
	 * 
	 * }
	 */

}
